@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic Morris
  *
- *   $Id: app.c,v 1.29 2003/11/27 20:25:12 dom Exp $
- *   $Date: 2003/11/27 20:25:12 $
+ *   $Id: app.c,v 1.31 2003/11/27 22:47:58 dom Exp $
+ *   $Date: 2003/11/27 22:47:58 $
  *
  *
  *   Implements the application
@@ -35,7 +35,9 @@ struct _app {
     gui_t     *gui;
     menu_t    *menu;
     char      *current_media;
-    char      *next_media;
+    int        num_next_media;
+    char     **next_media;
+    char       paused;
 };
 
 
@@ -86,6 +88,8 @@ static int           app_music(menu_t *menu,void *iapp, int sel);
 static int           process_music(menu_t *menu,void *iapp, int sel);
 static int           music_keys(void *ptr, int code);
 static int           music_ack(int acktype, void *param, unsigned char *buf, int len);
+static int           music_colour_keys(menu_t *menu, void *iapp, int key, int sel);
+
 
 static int           app_settings(menu_t *menu,void *iapp, int sel);
 static int           process_settings(menu_t *menu,void *iapp, int sel);
@@ -97,6 +101,9 @@ static dirlist_t    *new_dirlist(app_t *app);
 static void          delete_dirlist(void *dir);    /* Ahem */
 
 
+static void          next_media_clear(app_t *app);
+static void          next_media_pop(app_t *app);
+static void          next_media_push(app_t *app, char *url);
 
 
 
@@ -136,7 +143,10 @@ app_t *new_app(dongle_t *dongle, render_t *render, gui_t *gui)
     app->gui    = gui;
     app->dongle = dongle;
     app->current_media = NULL;
+    app->num_next_media = 0;
     app->next_media = NULL;
+    app->paused = FALSE;
+
 
     app_main(NULL,app,0);    
 
@@ -155,8 +165,7 @@ void delete_app(app_t *app)
     }
 
     if ( app->next_media ) {
-        free(app->next_media);
-        app->next_media = NULL;
+	next_media_clear(app);
     }
 
     free(app);
@@ -233,14 +242,10 @@ static int livetv_ack(int acktype, void *param, unsigned char *buf, int len)
     case RDC_STOP:             
         /* The client has sent a stop message, this could be response to "Stop" key press or
            us sending out a stop so we can switch to the net media file */ 
-        if ( app->next_media ) {
-            if ( app->current_media ) {
-                free(app->current_media);
-                app->current_media = NULL;
-            }
-            gui_send_play(app->gui,app->next_media);
-            app->current_media = app->next_media;
-            free(app->next_media);
+        if ( app->num_next_media ) {
+	    next_media_pop(app);           
+	    app->paused = FALSE;
+            gui_send_play(app->gui,app->current_media);
         } else {
             if ( ( stream_get_lasttype(app->dongle->stream) & MEDIA_MASK ) == MEDIA_MPEG ) {
                 gui_send_message(app->gui,&rfb_media_menu);
@@ -250,6 +255,7 @@ static int livetv_ack(int acktype, void *param, unsigned char *buf, int len)
                 free(app->current_media);
                 app->current_media = NULL;
             }
+	    app->paused = FALSE;
             /* Remove registered handlers */
             gui_register_keys(app->gui,REGISTER_MEDIA,NULL,app);
             gui_register_ack(app->gui,REGISTER_MEDIA,NULL,app);
@@ -324,12 +330,11 @@ static int process_liveradio(menu_t *menu,void *iapp, int sel)
     printf("Sending media do request %s\n",chan->url);
     if ( app->current_media == NULL ) {        
         app->current_media = strdup(chan->url);
+	app->paused = FALSE;
         gui_send_play(app->gui,chan->url);
     } else {
-        if ( app->next_media ) {
-            free(app->next_media);
-        }
-        app->next_media = strdup(chan->url);
+	next_media_clear(app);
+	next_media_push(app,chan->url);       
         gui_send_message(app->gui,&rfb_media_stop);
     }
     gui_register_keys(app->gui,REGISTER_MEDIA,liveradio_keys,app);
@@ -345,6 +350,7 @@ static int liveradio_keys(void *ptr, int code)
     switch ( code ) { 
     case kStop:
         gui_send_message(app->gui,&rfb_media_stop);
+	next_media_clear(app);
         menu_display(app->menu);
         break;
     case kVolUp:
@@ -368,19 +374,17 @@ static int liveradio_ack(int acktype, void *param, unsigned char *buf, int len)
 
     switch ( acktype ) {
     case RDC_STOP:     /* The client has sent a stop message */  
-        if ( app->next_media ) {
-            if ( app->current_media ) {
-                free(app->current_media);
-            }
-            gui_send_play(app->gui,app->next_media);
-            app->current_media = app->next_media;
-            app->next_media = NULL;
+        if ( app->num_next_media ) {
+	    next_media_pop(app);
+	    app->paused = FALSE;
+            gui_send_play(app->gui,app->current_media);
         } else {
             /* Remove registered handlers */
             if ( app->current_media ) {
                 free(app->current_media);
                 app->current_media = NULL;
             }
+	    app->paused = FALSE;
             gui_register_keys(app->gui,REGISTER_MEDIA,NULL,app);
             gui_register_ack(app->gui,REGISTER_MEDIA,NULL,app);
         }
@@ -440,6 +444,7 @@ static int app_music(menu_t *menu,void *iapp, int sel)
         }       
         menu_add(child,buf,process_music);
     }
+    menu_set_colour_actions(child,"Back","Play dir",NULL,NULL,music_colour_keys);
 
     menu_display(child);
     return 1;
@@ -468,7 +473,7 @@ static int process_music(menu_t *menu,void *iapp, int sel)
         dirlist2->filelist = directory_scan(buf,&dirlist2->filelist_num);
 
         child = new_menu(dirlist2->app->dongle,dirlist2->app->render,menu,dirlist2,0);
-        snprintf(buf,sizeof(buf),"Music - %s",dirlist2->path);
+        snprintf(buf,sizeof(buf),"Music - %s",entry->name);
         menu_set_title(child,buf);   
         menu_set_app_clearup(child,delete_dirlist);
             
@@ -480,18 +485,20 @@ static int process_music(menu_t *menu,void *iapp, int sel)
             }       
             menu_add(child,buf,process_music);
         }
+	menu_set_colour_actions(child,"Back","Play dir",NULL,NULL,music_colour_keys);
+
         menu_display(child);
         return 1;        
     } else {
 	snprintf(buf,sizeof(buf),"%s/%s",dirlist->path,entry->name);
         if ( dirlist->app->current_media == NULL ) {        
             dirlist->app->current_media = strdup(buf);
+	    dirlist->app->paused = FALSE;
             gui_send_play(dirlist->app->gui,buf);
         } else {
-            if ( dirlist->app->next_media ) {
-                free(dirlist->app->next_media);
-            }        
-            dirlist->app->next_media = strdup(buf);
+	    next_media_clear(dirlist->app);
+	    next_media_push(dirlist->app,buf);           
+	    dirlist->app->paused = FALSE;
             gui_send_message(dirlist->app->gui,&rfb_media_stop);    
         }
         gui_register_keys(dirlist->app->gui,REGISTER_MEDIA,music_keys,dirlist->app);
@@ -501,6 +508,45 @@ static int process_music(menu_t *menu,void *iapp, int sel)
     return 1;     /* Carry on displaying this menu */
 }
 
+static int music_colour_keys(menu_t *menu, void *iapp, int key, int sel)
+{
+    char           buf[FILENAME_MAX+1];
+    dirlist_t     *dir = iapp;
+    int            i;
+    char           play = TRUE;
+
+    switch ( key ) {
+    case kRed:
+	return 0;
+    case kGreen:       /* Add all contents of the directory */
+	if ( dir->app->current_media ) {
+	    play = FALSE;
+	}
+	next_media_clear(dir->app);
+
+	for ( i = 0; i < dir->filelist_num; i++ ) {
+	    if ( dir->filelist[i].flags == 0 ) {
+		snprintf(buf,sizeof(buf),"%s/%s",dir->path,dir->filelist[i].name);
+		next_media_push(dir->app,buf);
+	    }
+	}
+	if ( dir->app->num_next_media ) {
+	    if ( play ) {
+		next_media_pop(dir->app);
+		gui_send_play(dir->app->gui,dir->app->current_media);
+		gui_register_keys(dir->app->gui,REGISTER_MEDIA,music_keys,dir->app);
+		gui_register_ack(dir->app->gui,REGISTER_MEDIA,music_ack,dir->app);
+	    } else {
+		gui_send_message(dir->app->gui,&rfb_media_stop);
+	    }
+	}
+	break;
+    default:
+    }
+
+    return 1;     /* Carry on displaying this menu */
+}
+
 static int music_keys(void *ptr, int code)
 {
     app_t     *app = ptr;
@@ -510,6 +556,19 @@ static int music_keys(void *ptr, int code)
         gui_send_message(app->gui,&rfb_media_stop);
         menu_display(app->menu);
         break;
+    case kPlay:
+	gui_send_play(app->gui,app->current_media);
+	app->paused = FALSE;
+	break;
+    case kPause:
+	if ( app->paused ) {
+	    app->paused = FALSE;
+	    gui_send_play(app->gui,app->current_media);
+	} else {
+	    app->paused = TRUE;
+	    gui_send_message(app->gui,&rfb_media_pause);
+	}
+	break;
     case kVolUp:
         gui_send_message(app->gui,&rfb_media_volup);
         break;
@@ -519,6 +578,15 @@ static int music_keys(void *ptr, int code)
     case kMute:
         gui_send_message(app->gui,&rfb_media_mute);
         break;
+    case kFastFwd:
+	gui_send_message(app->gui,&rfb_media_forward);
+	break;
+    case kFastRew:
+	gui_send_message(app->gui,&rfb_media_rewind);
+	break;
+    case kSkip:
+	gui_send_message(app->gui,&rfb_media_stop);
+	break;
     default:
         return 0;      /* Carry on processing menu keys */
     }   
@@ -531,23 +599,25 @@ static int music_ack(int acktype, void *param, unsigned char *buf, int len)
 
     switch ( acktype ) {
     case RDC_STOP:     /* The client has sent a stop message */  
-        if ( app->next_media ) {
-            if ( app->current_media ) {
-                free(app->current_media);
-            }
-            gui_send_play(app->gui,app->next_media);
-            app->current_media = app->next_media;
-            app->next_media = NULL;
+        if ( app->num_next_media ) {
+	    next_media_pop(app);
+	    app->paused = FALSE;
+            gui_send_play(app->gui,app->current_media);          
         } else {
             /* Remove registered handlers */
             if ( app->current_media ) {
                 free(app->current_media);
                 app->current_media = NULL;
             }
+	    app->paused = FALSE;
             gui_register_keys(app->gui,REGISTER_MEDIA,NULL,app);
             gui_register_ack(app->gui,REGISTER_MEDIA,NULL,app);
         }
         break;
+    case RDC_FORWARD:
+    case RDC_REWIND:
+	gui_send_play(app->gui,app->current_media);
+	break;
     default:
         return 0;    /* Kick it up a level */
     }
@@ -804,6 +874,49 @@ static int read_channel_file(char *filename, int *num, channel_t **list,
     return 0;
 }
 
+static void next_media_clear(app_t *app)
+{
+    int         i;
+
+    if ( app->num_next_media ) {
+	for ( i = 0; i > app->num_next_media; i++ ) {
+	    free(app->next_media[i]);
+	}
+	free(app->next_media);
+	app->next_media = NULL;
+	app->num_next_media = 0;
+    }
+}
+
+static void next_media_pop(app_t *app)
+{
+    int       i;
+
+    if ( app->num_next_media ) {     /* Which it already is... */
+	if ( app->current_media ) {
+	    free(app->current_media);
+	}
+
+	app->current_media = app->next_media[0];
+
+	for ( i = 1; i < app->num_next_media; i++ ) {
+	    app->next_media[i-1] = app->next_media[i];
+	}
+	if ( --app->num_next_media == 0 ) {
+	    free(app->next_media);
+	    app->next_media = NULL;
+	}
+    } 
+}
+
+static void next_media_push(app_t *app, char *url)
+{
+    int       i = app->num_next_media++;
+
+    app->next_media = realloc(app->next_media,app->num_next_media * sizeof(char *));
+    app->next_media[i] = strdup(url);
+}
+
 
 
 /*
