@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic Morris
  *
- *   $Id: gui.c,v 1.24 2003/11/25 10:32:22 dom Exp $
- *   $Date: 2003/11/25 10:32:22 $
+ *   $Id: gui.c,v 1.25 2003/11/27 22:47:58 dom Exp $
+ *   $Date: 2003/11/27 22:47:58 $
  *
  *
  *   Handles the GUI...
@@ -628,7 +628,7 @@ void gui_send_play(gui_t *gui, char *name)
 
     memcpy(buf,msg->msg,msg->len);
 
-    buf[8] = len;
+    buf[8] = len;        /* What happens here if path > 256?!?! */
 
     strcpy(buf + msg->len,name);
 
