@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic Morris
  *
- *   $Id: menu.c,v 1.19 2003/11/27 20:25:13 dom Exp $
- *   $Date: 2003/11/27 20:25:13 $
+ *   $Id: menu.c,v 1.21 2003/11/27 22:47:59 dom Exp $
+ *   $Date: 2003/11/27 22:47:59 $
  *
  *
  *   Constructs a menu and all such things
@@ -56,9 +56,11 @@ struct _menu {
     char          *green;
     char          *yellow;
     char          *blue;
-    int          (*colour_func)(menu_t *menu,void *app, int sel);
+    int          (*colour_func)(menu_t *menu,void *app, int key, int sel);
 };
 
+#define MENU_ROWS(menu)  ((menu)->maxrows -  3)
+
 static int        menu_keys(void *ptr, int code);
 
 
@@ -213,7 +215,8 @@ void menu_display(menu_t *menu)
 
 
     /* Calculate how many rows to print */
-    last = (menu->top_offset + (menu->maxrows - 2) < menu->numoptions) ? menu->top_offset + (menu->maxrows - 2) : menu->numoptions;
+    last = (menu->top_offset + MENU_ROWS(menu) < menu->numoptions) 
+	? menu->top_offset + MENU_ROWS(menu) : menu->numoptions;
     for ( row = 2, i = menu->top_offset; i < last; i++, row++ ) {
         if ( menu->options[i].type == MENU_INT ) {
             printf("option %d val %d\n",i,*((int *)menu->options[i].value_ptr));
@@ -245,6 +248,7 @@ static int menu_keys(void *ptr, int code)
     menu_t    *menu = ptr;
     int        pos;
     int      (*func)(menu_t *,void *app, int sel);
+    int      (*cfunc)(menu_t *,void *app, int key,int sel);
     option_t  *option;
     unsigned int       *num;
     int      streamtype = 0;
@@ -254,7 +258,6 @@ static int menu_keys(void *ptr, int code)
         streamtype = stream_get_type(menu->dongle->stream);
     }
 
-    printf("Got menu key %d\n",code);
 
     /* If we're in streaming mpeg mode, then we'd like to change channels
        without going back to the menu 
@@ -264,7 +267,7 @@ static int menu_keys(void *ptr, int code)
         case kChanUp:
             if ( menu->current_selection < menu->numoptions - 1 ) {
                 menu->current_selection++;
-                if ( menu->top_offset + (menu->maxrows - 2) <= menu->current_selection ) {
+                if ( menu->top_offset + MENU_ROWS(menu) <= menu->current_selection ) {
                     menu->top_offset++;
                 }
             } else {
@@ -288,8 +291,8 @@ static int menu_keys(void *ptr, int code)
                 }
             } else {
                 menu->current_selection = menu->numoptions - 1;          
-                if ( menu->numoptions > menu->maxrows - 2 ) {
-                    menu->top_offset = menu->numoptions - ( menu->maxrows - 2);
+                if ( menu->numoptions > MENU_ROWS(menu) ) {
+                    menu->top_offset = menu->numoptions - MENU_ROWS(menu);
                 }
             }
             func = menu->options[menu->current_selection].callback_func;
@@ -316,8 +319,8 @@ static int menu_keys(void *ptr, int code)
                 }
             } else {
                 menu->current_selection = menu->numoptions - 1;          
-                if ( menu->numoptions > menu->maxrows - 2 ) {
-                    menu->top_offset = menu->numoptions - ( menu->maxrows - 2);
+                if ( menu->numoptions > MENU_ROWS(menu) ) {
+                    menu->top_offset = menu->numoptions - MENU_ROWS(menu);
                 }
             }
             
@@ -328,7 +331,7 @@ static int menu_keys(void *ptr, int code)
         if ( streamtype != MEDIA_MPEG ) {
             if ( menu->current_selection < menu->numoptions - 1 ) {
                 menu->current_selection++;
-                if ( menu->top_offset + (menu->maxrows - 2) <= menu->current_selection ) {
+                if ( menu->top_offset + MENU_ROWS(menu) <= menu->current_selection ) {
                     menu->top_offset++;
                 }
             } else {
@@ -350,7 +353,7 @@ static int menu_keys(void *ptr, int code)
             }
         }
         break;
-    case kVolDn:
+    case kVolUp:
         option = &menu->options[menu->current_selection];
         if ( option->type == MENU_INT ) {
             menu_t *parent = menu->parent;
@@ -365,7 +368,7 @@ static int menu_keys(void *ptr, int code)
   
         }
         break;
-    case kVolUp:
+    case kVolDn:
         option = &menu->options[menu->current_selection];
         if ( option->type == MENU_INT ) {
             menu_t *parent = menu->parent;
@@ -392,6 +395,25 @@ static int menu_keys(void *ptr, int code)
             menu_display(parent);
         }
         break;
+    case kSkip:        /* Go down a page */
+	if ( menu->current_selection + MENU_ROWS(menu) > ( menu->numoptions - MENU_ROWS(menu) ) ) {
+	    menu->current_selection = menu->top_offset = menu->numoptions - MENU_ROWS(menu);
+	    if ( menu->current_selection < 0 ) {
+		menu->current_selection = menu->top_offset = 0;
+	    }
+	} else {
+	    menu->current_selection = menu->top_offset =  menu->current_selection + MENU_ROWS(menu);
+	}
+	menu_display(menu);
+	break;
+    case kReplay:       /* Go up a page */
+	if ( menu->current_selection - MENU_ROWS(menu) < 0 ) {
+	    menu->current_selection = menu->top_offset = 0;
+	} else {
+	    menu->current_selection = menu->top_offset =  menu->current_selection - MENU_ROWS(menu);
+	}
+	menu_display(menu);
+	break;
     case kGo:
         while ( menu->parent ) {
             menu_t *parent = menu->parent;
@@ -425,8 +447,8 @@ static int menu_keys(void *ptr, int code)
         }
         break;
     case kRed ... kBlue:
-        func = menu->colour_func;        
-        if ( func && func(menu,menu->app,code - kRed) == 0 ) {
+        cfunc = menu->colour_func;        
+        if ( cfunc && cfunc(menu,menu->app,code,menu->current_selection) == 0 ) {
 	    menu_t *parent;
             parent = menu->parent;
             delete_menu(menu);
@@ -456,7 +478,7 @@ void menu_set_app_clearup(menu_t *menu, void (*clearup)(void *ptr))
 
 /** \brief Set what the colour keys do 
  *
- *  \param menu Menu handle
+ *  \param menu    Menu handle
  *  \param red     Red button text
  *  \param green   Green button text
  *  \param yellow  Yellow button text
@@ -464,33 +486,50 @@ void menu_set_app_clearup(menu_t *menu, void (*clearup)(void *ptr))
  *  \param func    Callback function - called back with 0 - 3 for the colour button that has been
  *                 pressed
  */
-void menu_set_colactions(menu_t *menu,
-                         char   *red,
-                         char   *green,
-                         char   *yellow,
-                         char   *blue,
-                         int   (*func)(menu_t *menu,void *app, int sel))
+void menu_set_colour_actions(menu_t *menu,
+			     char   *red,
+			     char   *green,
+			     char   *yellow,
+			     char   *blue,
+			     int   (*func)(menu_t *menu,void *app, int key, int sel))
 {
 
     if ( menu->red ) {
         free(menu->red);
     }
-    menu->red = strdup(red);
+    if ( red ) {
+	menu->red = strdup(red);
+    } else {
+	menu->red = NULL;
+    }
 
     if ( menu->green ) {
         free(menu->green);
     }
-    menu->green = strdup(green);
+    if ( green ) {
+	menu->green = strdup(green);
+    } else {
+	menu->green = NULL;
+    }
 
     if ( menu->yellow ) {
         free(menu->yellow);
+    } 
+
+    if ( yellow ) {
+	menu->yellow = strdup(yellow);
+    } else {
+	menu->yellow = NULL;
     }
-    menu->yellow = strdup(yellow);
 
     if ( menu->blue ) {
         free(menu->blue);
     }
-    menu->blue = strdup(blue);
+    if ( blue ) {
+	menu->blue = strdup(blue);
+    } else {
+	menu->blue = NULL;
+    }
 
     menu->colour_func = func;
 }
